/**************************************************************************
 OmegaT - Computer Assisted Translation (CAT) tool
          with fuzzy matching, translation memory, keyword search,
          glossaries, and translation leveraging into updated projects.

 Copyright (C) 2010 Yasutaka ATARASHI
               Home page: http://www.omegat.org/
               Support center: http://groups.yahoo.com/group/OmegaT/

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **************************************************************************/

package org.omegat.filters2.mediawiki;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.omegat.filters2.AbstractFilter;
import org.omegat.filters2.Instance;
import org.omegat.filters2.TranslationException;
import org.omegat.util.Log;

/**
 * Filter for support MediwWiki import files.
 * 
 * @author Yasutaka ATARASHI
 */
public class MediaWikiFilter extends AbstractFilter {

    public String getFileFormatName() {
        return "MediaWiki import files";
    }

    public Instance[] getDefaultInstances() {
        return new Instance[] { new Instance("*.UTF8") };
    }

    public boolean isSourceEncodingVariable() {
        return true;
    }

    public boolean isTargetEncodingVariable() {
        return true;
    }

    protected static final Pattern RE_HEADING = Pattern.compile("^(=+)(.*)\\1\\s*$");
    private void processHeading(String specifier, String title, BufferedWriter outFile) throws IOException {
		outFile.write(specifier + processEntry(title) + specifier);
		outFile.newLine();
    }

    protected static final Pattern RE_ENUMERATE = Pattern.compile("^([*#]+\\s*)(.*)$");
    private void processEnumerate(String specifier, String text, BufferedWriter outFile) throws IOException {
    	outFile.write(specifier + processEntry(text));
        outFile.newLine();
    }

    // TODO: fix naive expression
    protected static final Pattern RE_SOURCETAG = Pattern.compile("(<source(?:[^>]*?lang=\"([^\"]+)\"|)[^>]*>|</source>)");
    protected static final int RE_SOURCETAG_GROUP_TAG = 1;
    protected static final int RE_SOURCETAG_GROUP_LANG = 2;
    private static enum SourceType { NONE, CPP, C, UNKNOWN };
    protected static final Pattern RE_C_SOURCE = Pattern.compile("/\\*+\\s*(.*?[^\\s*])\\s*\\*+/|\"([^\"]*)\"", Pattern.DOTALL);
    protected static final Pattern RE_CPP_SOURCE = Pattern.compile("//\\s*([^\\n]*\\S)\\s*\\n|/\\*+\\s*(.*?[^\\s*])\\s*\\*+/|\"([^\"]*)\"", Pattern.DOTALL);
    private void processSourceTag(String s, SourceType st, BufferedWriter outFile) throws IOException {
		Matcher m = null;
		if(st == SourceType.C) {
			m = RE_C_SOURCE.matcher(s);
		} else if (st == SourceType.CPP) {
			m = RE_CPP_SOURCE.matcher(s);
		} else {
			outFile.write(processEntry(s));
			return;
		}

		int prevEnd = 0;
		if(m.find(s.indexOf('>')+1)) { // Skip open tag
			do {
				for(int i = 1; i <= m.groupCount(); ++i) {
					if(m.group(i) != null) {
						outFile.write(processEntry(s.substring(prevEnd, m.start(i))));
						outFile.write(processEntry(s.substring(m.start(i), m.end(i))));
						prevEnd = m.end(i);
						break;
					}
				}
			} while(m.find());
		}
		outFile.write(processEntry(s.substring(prevEnd)));
    }
    
    private void processParagraph(String s, BufferedWriter outFile) throws IOException {
    	outFile.write(processEntry(s));
    }
    
    protected void processFile(BufferedReader inFile, BufferedWriter outFile)
            throws IOException, TranslationException {

        String s;
        String sTarget = null;
        SourceType st = SourceType.NONE;
        while ((s = inFile.readLine()) != null) {
        	// TODO: Extract to interface table
        	Matcher mHeading = RE_HEADING.matcher(s);
        	Matcher mEnumerate = RE_ENUMERATE.matcher(s);
        	Matcher mSourceTag = RE_SOURCETAG.matcher(s);
        	if (mHeading.matches()) {
        		if(outFile != null && sTarget != null) {
                    processParagraph(sTarget, outFile);
        			sTarget = null;
        		}
        		processHeading(mHeading.group(1), mHeading.group(2), outFile);
        	} else if (mEnumerate.matches()) {
        		if(outFile != null && sTarget != null) {
                    processParagraph(sTarget, outFile);
        			sTarget = null;
        		}
        		processEnumerate(mEnumerate.group(1), mEnumerate.group(2), outFile);
           	} else if(mSourceTag.find()) {
           		int nPrevStart = 0;
           		int nPrevEnd = 0;
           		do {
               		if(mSourceTag.group(RE_SOURCETAG_GROUP_TAG).equals("</source>")) { // close tag
               			if(sTarget == null) {
               				sTarget = s.substring(nPrevStart, mSourceTag.end());
               			} else {
               				sTarget += s.substring(nPrevStart, mSourceTag.end());
               			}
               			processSourceTag(sTarget, st, outFile);
               			sTarget = null;
               			st = SourceType.NONE;
               		} else { // open tag
               			if(sTarget == null) {
               				sTarget = s.substring(nPrevEnd, mSourceTag.start());
               			} else {
               				sTarget += s.substring(nPrevEnd, mSourceTag.start());
               			}
               			processParagraph(sTarget, outFile);
               			sTarget = null;
               			if(mSourceTag.group(RE_SOURCETAG_GROUP_LANG) != null && mSourceTag.group(RE_SOURCETAG_GROUP_LANG).equals("cpp")) {
               				st = SourceType.CPP;
               			} else if(mSourceTag.group(RE_SOURCETAG_GROUP_LANG) != null && mSourceTag.group(RE_SOURCETAG_GROUP_LANG).equals("c")) {
               				st = SourceType.C;
               			} else {
               				st = SourceType.UNKNOWN;
               			}
               		}
               		nPrevStart = mSourceTag.start();
               		nPrevEnd = mSourceTag.end();
               	} while(mSourceTag.find());
           		if(st == SourceType.NONE) {
           			sTarget = s.substring(nPrevEnd) + "\n";
           		} else {
           			sTarget = s.substring(nPrevStart) + "\n";
           		}
           	} else if (outFile != null && sTarget != null && s.length() == 0 && st == SourceType.NONE) { // blank line
                processParagraph(sTarget + s + "\n", outFile);
                sTarget = null;
            } else {
            	if(sTarget == null) {
            		sTarget = s + "\n";
            	} else {
            		sTarget = sTarget + s + "\n";
            	}
            }
        }
    	if (outFile != null && sTarget != null) {
            processParagraph(sTarget, outFile);
    	}
    }

}
